@@ -202,18 +202,19 @@ g_once_impl (GOnce       *once,
 gboolean
 g_once_init_enter_impl (volatile gsize *value_location)
 {
-  gboolean need_init;
+  gboolean need_init = FALSE;
   g_mutex_lock (g_once_mutex);
-  if (!g_once_init_list || !g_slist_find (g_once_init_list, (void*) value_location))
+  if (g_atomic_pointer_get ((void**) value_location) == NULL)
     {
-      g_once_init_list = g_slist_prepend (g_once_init_list, (void*) value_location);
-      need_init = TRUE;
-    }
-  else
-    {
-      while (g_slist_find (g_once_init_list, (void*) value_location))
-        g_cond_wait (g_once_cond, g_once_mutex);
-      need_init = FALSE;
+      if (!g_slist_find (g_once_init_list, (void*) value_location))
+        {
+          need_init = TRUE;
+          g_once_init_list = g_slist_prepend (g_once_init_list, (void*) value_location);
+        }
+      else
+        do
+          g_cond_wait (g_once_cond, g_once_mutex);
+        while (g_slist_find (g_once_init_list, (void*) value_location));
     }
   g_mutex_unlock (g_once_mutex);
   return need_init;
@@ -223,7 +224,7 @@ void
 g_once_init_leave (volatile gsize *value_location,
                    gsize           initialization_value)
 {
-  g_return_if_fail (g_atomic_pointer_get ((void**) value_location) == 0);
+  g_return_if_fail (g_atomic_pointer_get ((void**) value_location) == NULL);
   g_return_if_fail (initialization_value != 0);
   g_return_if_fail (g_once_init_list != NULL);
 
@@ -496,10 +497,10 @@ g_static_private_set (GStaticPrivate *private_key,
 void
 g_static_private_free (GStaticPrivate *private_key)
 {
-  guint index = private_key->index;
+  guint idx = private_key->index;
   GRealThread *thread;
 
-  if (!index)
+  if (!idx)
     return;
 
   private_key->index = 0;
@@ -512,27 +513,27 @@ g_static_private_free (GStaticPrivate *private_key)
       GArray *array = thread->private_data;
       thread = thread->next;
 
-      if (array && index <= array->len)
+      if (array && idx <= array->len)
 	{
 	  GStaticPrivateNode *node = &g_array_index (array,
 						     GStaticPrivateNode,
-						     index - 1);
+						     idx - 1);
 	  gpointer ddata = node->data;
 	  GDestroyNotify ddestroy = node->destroy;
 
 	  node->data = NULL;
 	  node->destroy = NULL;
 
-	  if (ddestroy)
-	    {
-	      G_UNLOCK (g_thread);
-	      ddestroy (ddata);
-	      G_LOCK (g_thread);
-	      }
+          if (ddestroy)
+            {
+              G_UNLOCK (g_thread);
+              ddestroy (ddata);
+              G_LOCK (g_thread);
+            }
 	}
     }
   g_thread_free_indeces = g_slist_prepend (g_thread_free_indeces,
-					   GUINT_TO_POINTER (index));
+					   GUINT_TO_POINTER (idx));
   G_UNLOCK (g_thread);
 }
 
@@ -579,7 +580,7 @@ g_thread_cleanup (gpointer data)
 
 	  /* Just to make sure, this isn't used any more */
 	  g_system_thread_assign (thread->system_thread, zero_thread);
-	  g_free (thread);
+          g_free (thread);
 	}
     }
 }
@@ -637,13 +638,13 @@ g_thread_create_proxy (gpointer data)
 }
 
 GThread*
-g_thread_create_full (GThreadFunc 		 func,
-		      gpointer 		 data,
-		      gulong 		 stack_size,
-		      gboolean 		 joinable,
-		      gboolean 		 bound,
-		      GThreadPriority 	 priority,
-		      GError                **error)
+g_thread_create_full (GThreadFunc       func,
+		      gpointer          data,
+		      gulong            stack_size,
+		      gboolean          joinable,
+		      gboolean 	        bound,
+		      GThreadPriority   priority,
+		      GError          **error)
 {
   GRealThread* result;
   GError *local_error = NULL;
@@ -662,8 +663,11 @@ g_thread_create_full (GThreadFunc 		 func,
   G_THREAD_UF (thread_create, (g_thread_create_proxy, result,
 			       stack_size, joinable, bound, priority,
 			       &result->system_thread, &local_error));
-  result->next = g_thread_all_threads;
-  g_thread_all_threads = result;
+  if (!local_error)
+    {
+      result->next = g_thread_all_threads;
+      g_thread_all_threads = result;
+    }
   G_UNLOCK (g_thread);
 
   if (local_error)
@@ -970,5 +974,18 @@ g_thread_foreach (GFunc    thread_func,
     }
 }
 
+/**
+ * g_thread_get_initialized
+ *
+ * Indicates if g_thread_init() has been called.
+ *
+ * Since: 2.20
+ */
+gboolean
+g_thread_get_initialized ()
+{
+  return g_thread_supported ();
+}
+
 #define __G_THREAD_C__
 #include "galiasdef.c"
